from graphics import Graphics
import industries
import tile
from levels import level


from world import World
from rail import Rail
from rail import Station
from route import Route, Section
#from gamestates.buildrailstate import BuildRailState

X, Y = 0, 1

class Game:
	def __init__(self, graphics, player):
		
		self.graphics = graphics
		self.gui = None
		self.player = player
		self.timeLeft = 100
		
		self.world = World()
		self.graphics.world = self.world
		self.graphics.generateLevel()
		
		self.state = "BuildRails"
		self.selectedIndustry = None
		#self.gameStates = {"BuildRails": BuildRailState()}
		
		self.level = level.Level1()
		for kind, position in self.level.industries:
			self.createIndustry(kind, position)
				
		for kind, position in self.level.otherTiles:
			if kind == "Mountain":
				x,y = position
				self.world.map[(x,y)]     = tile.Mountain((x,y))
				self.world.map[(x,y+1)]   = tile.Mountain((x,y+1))
				self.world.map[(x+1,y)]   = tile.Mountain((x+1,y))
				self.world.map[(x+1,y+1)] = tile.Mountain((x+1,y+1))
				self.graphics.placeNewHex((x,y), kind)
				self.graphics.placeNewHex((x,y+1), kind)
				self.graphics.placeNewHex((x+1,y), kind)
				self.graphics.placeNewHex((x+1,y+1), kind)
			if kind == "River":
				x,y = position
				self.world.map[(x,y)]     = tile.Water((x,y))
				self.world.map[(x,y+1)]   = tile.Water((x,y+1))
				self.world.map[(x,y+2)]   = tile.Water((x,y+2))
				self.world.map[(x+1,y+3)]   = tile.Water((x+1,y+3))
				self.graphics.placeNewHex((x,y), "Water")
				self.graphics.placeNewHex((x,y+1), "Water")
				self.graphics.placeNewHex((x,y+2), "Water")
				self.graphics.placeNewHex((x+1,y+3), "Water")
				
		
	
	def handleMouseClick(self, position):
		
		### For later beautification
		#self.gameStates[self.state].handleMouseClick(position)
		
		tile = self.world.map[position]
		
		### If player clicks on the ground, a rail should be built.
		if tile.isGround and tile.isBuildable:
			neighbours = self.world.map.getNeighbours(position)

			neighbourIndustries = [tile for tile in neighbours if tile.isIndustry]
			neighbourRails = [tile for tile in neighbours if tile.kind == "Rail" or 
																  tile.kind == "Station"]			

			### If there are more than two "rails" near it, build a station instead.
			if len(neighbourRails) + len(neighbourIndustries) > 2:
				self.createRailroad(position, "Station")
				self.player.money -= 500
			elif len(neighbours) != 0:
				self.createRailroad(position)
				self.player.money -= 500

				
		elif tile.isIndustry:
			
			if tile.owner is None and self.player.money > tile.buyCost and \
					len(self.player.industries) < sum([c.size for c in self.player.cities]):
				tile.owner = self.player
				self.player.money -= tile.buyCost
				self.player.industries.append(tile)
				self.gui.placeStationIcon(tile)
				return
				
			if tile.owner is None:
				return
			if self.selectedIndustry is None:
				if tile.kind != "City":
					self.selectedIndustry = tile
			else:
				self.setupRoute(self.selectedIndustry, tile)


	def setupRoute(self, sourceTile, targetTile):
		print "##################################"
		sourceTile.neighbours = self.world.map.getNeighbours(sourceTile.position)
		found, dist = self.canReachIndustry(sourceTile.neighbours, targetTile, [])
		if found:
			print "industry",tile,"found", dist
			if sourceTile.producedResource in targetTile.requirements.keys():
				if self.selectedIndustry.targetIndustry is not None:
					sourceTile.targetIndustry.incoming[sourceTile.producedResource] -= 1
					self.player.routes.remove((sourceTile, sourceTile.targetIndustry))
				
				targetTile.incoming[sourceTile.producedResource] += 1
				self.player.routes.append((sourceTile, targetTile))
				sourceTile.distanceToTarget = dist						
				sourceTile.targetIndustry = targetTile
		else:
			print "industry",tile," not found", dist
		self.gui.update()
		self.graphics.render()
		self.selectedIndustry = None	
		
		
	def handleRightMouseClick(self, position):
		tile = self.world.map[position]
		
		if tile.kind == "City":
			costToUpgrade = 1000 * tile.size**2
			if self.player.money > costToUpgrade:
				tile.size += 1 
				self.player.money -= costToUpgrade
		else:
			self.selectedIndustry = None
			self.gui.update()
			self.graphics.render()
		
	def update(self):
		for sourceInd, destInd in self.player.routes:
			self.player.money -= sourceInd.distanceToTarget * 10
			for res, amount in destInd.incoming.items():
				self.player.money += amount * destInd.purchaseValue

	
	def canReachIndustry(self, tiles, industry, parsedTiles, dist=0):
		found = False
		for tile in tiles:
			if tile not in parsedTiles: 
				parsedTiles.append(tile)
			else:
				continue
				
			if tile is industry:
				return True, dist
			else:
				neighbours = [n for n in self.world.map.getNeighbours(tile.position) if not n.isGround]
				recFound, recDist = self.canReachIndustry(neighbours, industry, parsedTiles, dist+1)
				if recFound:
					return recFound, recDist
				#found = found or self.canReachIndustry(neighbours, industry, parsedTiles)
		return (found, dist)
	

	
	
	
	def createRailSection(self, section, tile):
		if tile.isBuilding: 
			return

		if tile not in section.rails:
			section.rails.append(tile)
			
		for n in [t for t in tile.neighbourRails if t not in section.rails]:
			self.createRailSection(section, n)

	def createIndustry(self, industryType, position):
		"""
		Creates an industry at position. 
		"""	
		
		### Do magic industryType => className conversion
		### Leave this to Jonex ;)
		
		### For now, hardcoded
		### Create a coal mine and update the gui
		industry = None
		if industryType == "Coal mine":
			industry = industries.CoalMine(position)
		elif industryType == "Iron mine":
			industry = industries.IronMine(position)
		elif industryType == "Steel mill":
			industry = industries.SteelMill(position)
		elif industryType == "City":
			industry = industries.City(position)
			self.player.cities.append(industry)
			industry.owner = self.player
		elif industryType == "Farm":
			industry = industries.Farm(position)
		elif industryType == "Tool factory":
			industry = industries.ToolFactory(position)
			
		if industry == None:
			print "Can't build", industryType
			return

			
		self.world.industries.append(industry)
		self.world.industryPositions[position] = industry
		self.world.map.tilemap[position[X]][position[Y]] = industry
		self.graphics.placeNewHex(industry.position, industry.kind)
		
	def createRailroad(self, position, kind="Rail"):
		"""
		Creates a railroad at position. 
		"""

		if kind == "Rail":
			rail = Rail(position)
			print rail.position, "NEW RAIL"
			self.world.map[position] = rail
			self.graphics.updateNewRail(rail, first=True)
		elif kind == "Station":
			station = Station(position)
			print station.position, "NEW STATION!"
			self.world.map[position] = station
			self.graphics.updateNewRail(station, first=True)
			self.graphics.placeNewHex(station.position, station.kind)
			

		"""
		DONT DELETE
		self.graphics.placeNewHex((position[X] - 1, position[Y] - 1), 19) #above
		self.graphics.placeNewHex((position[X]	, position[Y] - 1), 19) #over, right
		self.graphics.placeNewHex((position[X] + 1, position[Y]	), 19) #below, right
		self.graphics.placeNewHex((position[X] + 1, position[Y] + 1), 19) #below
		self.graphics.placeNewHex((position[X]	, position[Y] + 1), 19) #below, left
		self.graphics.placeNewHex((position[X] - 1, position[Y]	), 19) #over, left
		"""

		
		
		
		
	############### OLD STUFF
		"""
		### Otherwise, the player has clicked on a rail, a station or an industry
		### This likely means that a new route should be created.
		### For more detailed info on how this shuold work, read
		### http://swecsteam.srekel.net/wiki/pmwiki.php?n=Game.UserInterface
		elif self.player.selectedRoute is not None:
			route = self.player.selectedRoute
			tile = self.world.map[position]
			
			### First check if the current route has a start point
			### If not, it should be placed (if the click was on an industry)
			if route.startPoint is None:
				if tile.isIndustry:
					route.startPoint = tile
					route.loadPoints.append((tile, 1))
					
			### Only if there already is a startPoint, is anything else interesting
			else:
				### If the clicked tile is an industry, check if it is directly reachable
				### from the last rail segment, or if it was the last one clicked.
				### If it is an "ok" industry, the "train" loads up cargo there
				### TODO: Add capacity check
				if tile.isIndustry:
					if tile == route.loadPoints[-1][0]:
						route.loadPoints[-1] = (tile, route.loadPoints[-1][1] + 1)
					elif route.sections != []:
						lastSection = route.sections[-1]
						endRails = [r for r in lastSection.rails 
											if len([n for n in r.neighbours 
															   if n.isBuilding]) > 0]

						if self.canReachIndustry(endRails, tile, []):
							if route.loadPoints[-1][0] == tile:
								route.loadPoints[-1] = (tile, route.loadPoints[-1][1] + 1)
							else:
								route.loadPoints.append((tile, 1))
						else:
							print "could not reach"
						
				### If the clicked tile is a rail, add it to the list of rail segments
				### Unless it already is a part of the last rail segment in the route
				elif tile.kind == "Rail":
					if (route.sections != [] and tile not in route.sections[-1].rails) or \
						 route.sections == []:
						section = Section([])
						self.createRailSection(section, tile)
					

						#for n in section.rails:
						#	print n.position, n.kind
						route.sections.append(section)
			print "HMC:", route.id
			print route.sections
			print route.loadPoints

	def canReachIndustry(self, tiles, industry, parsedTiles):
		found = False
		for tile in tiles:
			if tile not in parsedTiles: 
				parsedTiles.append(tile)
			else:
				continue
				
			if tile is industry:
				found = True
			else:
				neighbours = [n for n in self.world.map.getNeighbours(tile.position) if not n.isGround]
				found = found or self.canReachIndustry(neighbours, industry, parsedTiles)
		return found
	"""

		