import random

import pygame
from pygame.locals import *

from pgu import gui
from pgu import hexvid

from rail import Station

TOP, TOPRIGHT, BOTTOMRIGHT, BOTTOM, BOTTOMLEFT, TOPLEFT = 0, 1, 2, 3, 4, 5

TILEMAP_MIN = 1
TILEMAP_MAX = 28
		
class Graphics:
	def __init__(self, w, h, player):

		self.w = w
		self.h = h
		self.player = player

		### Set up tile kind => tile value mapping
		self.tileValues = {"Mountain":24, 
						   "Water":2, 
						   "Grass ground":5, 
						   "Dirt grass ground":6,
						   "Tree grass ground":7,
						   "Station":4, 
						   "Coal mine":9,
						   "Iron mine":10,
						   "Farm":11,
						   "Steel mill":12,
						   "City":13,
						   "Tool factory":14}
		
		### Set up basic pgu/pygame stuff
		self.app = gui.App()
		self.g = hexvid.Hexvid()
		self.g.screen = pygame.display.set_mode((w, h), SWSURFACE)
		

		### Load background
		self.g.load_background("../media/background1.png")

		### Set title
		pygame.display.set_caption("Steam Empire")

		### Load image data (not sure if level is necessary)
		TileWidth,TileHeight = 41,32
		self.g.tga_load_tiles('../media/railtiles.png', (TileWidth,TileHeight))
		self.g.tga_load_level('../media/hexlevel.tga')

		### Adjust the view so it's centered (doesn't work perfectly)
		self.g.view.x -= w / 2 
		self.g.view.y -= h / 2

		### Create a random play field
		#self.generateLevel()
	
	def scroll(self, x, y):
		self.g.view.x += x
		self.g.view.y += y
		#print "scrolling ", self.g.view.x, self.g.view.y
		
		
	def generateLevel(self):
		### Make black surrounding tiles
		for x in range(32):
			for y in range(32):
				self.g.set((x,y), 0)	

		
		### Make green grass tiles
		for x in range(TILEMAP_MIN, TILEMAP_MAX):
			for y in range(TILEMAP_MIN, TILEMAP_MAX):
				self.placeNewHex((x,y), self.world.map[(x-1,y-1)].kind)
				#self.g.set((x,y), 1)
			
		### Make some mountains
		#~ for x in range(2, 30):
			#~ for y in range(2, 30):
				#~ if random.random() < 0.04:
					#~ self.g.set((x, y), 5)
					#~ self.g.set((x+1, y), 5)
					#~ self.g.set((x, y+1), 5)
					#~ self.g.set((x+1,y+1), 5)
		
	def drawBackground(self):
		bw = self.background.get_width()
		bh = self.background.get_height()
		nx = int(self.w / bw) + 2
		ny = int(self.h / bh) + 2
		for x in range(nx):
			for y in range(ny):
				self.g.screen.blit(self.background, (x*bw - self.g.view.x % bw, 
								   y*bh - self.g.view.y % bh)) 
		
	def render(self):
		### Some calls I'm not 100% sure on what they do, but they're supposed
		### to be there. :)
		
		self.g.screen.fill((0,0,0)) # clear the screen
		
		
		"""
		for tile in self.world.map:
			if tile.kind == "Tile":
				self.g.set(tile.position, 24)
			if tile.kind == "Rail":
				self.g.set(tile.position, 64+self.railTileValue(tile))
		"""
		self.g.loop()		
		
		
		self.g.paint(self.g.screen)
		#self.drawBackground()
		
		#updates = self.g.update(self.g.screen)
		#pygame.display.update(updates)
		self.app.paint(self.g.screen)
		
		pygame.display.flip()
		
	def placeNewHex(self, position, tileKind):
		### Places new hex.
		# TODO: fix so it doesn't overwrite... ?
		
		#print "Placing hex with kind", tileKind, "at", position

		self.g.set(position, self.tileValues[tileKind])
		
		
	def getTileFromScreenCoords(self, position):
		### "position" is the mouse coordinates when clicked.
		#print position, " <==> ", self.g.screen_to_tile(position)
		return self.g.screen_to_tile(position)
		

	def updateNewRail(self, rail, first=False):
		#neighbours = self.world.map.getNeighbours(rail.position)
		#print rail.position, [n.kind for n in neighbours]

		railWasUpdated = self.updateTileValue(rail)
		#print rail.position, "railWasUpdated", railWasUpdated, rail.kind

		if railWasUpdated or first:
			#print rail.position, "rail.neighbourRails", rail.neighbourRails

			for neighbour in rail.neighbours:
				#print neighbour.position, "to be updated"
				self.updateNewRail(neighbour)
				
				
	def railTileValue(self, rail):
		neighbours = self.world.map.getNeighbours(rail.position)
		return sum(2**i for i in range(6) if neighbours[i].kind == "Rail" or
												neighbours[i].kind == "Station")


	def updateTileValue(self, rail, recurse=True):
		#print rail.position, "uTV", rail.kind
		oldVal = rail.tileValue
		rail.neighbours = self.world.map.getNeighbours(rail.position)
		rail.neighbourRails = [tile for tile in rail.neighbours if tile.kind == "Rail" or
							   										tile.kind == "Station"]
		rail.neighbourBuildings  = [tile for tile in rail.neighbours if tile.isBuilding]					   										
		rail.neighbourIndustries = [tile for tile in rail.neighbours if tile.isIndustry]
		
		if rail.kind == "Rail":
	
			if len(rail.neighbourRails) + len(rail.neighbourIndustries) > 2:
				station = Station(rail.position)
				station.neighbours = rail.neighbours
				station.neighbourRails = rail.neighbourRails
				station.neighbourIndustries = rail.neighbourIndustries
				station.tileValue = 4
				self.world.map[rail.position] = station
				self.placeNewHex(station.position, station.kind)
				return True
			
			
			
			if rail.neighbourBuildings != []:
				for n in rail.neighbours: 
					if recurse:	self.updateTileValue(n, recurse=False)
				### Find out if either of the surrounding rails go to the same industry
				### If they do, they are not interesting.
				#rail.neighbourRails = [tile for tile in rail.neighbours if tile.kind == "Rail"]
				okTiles = rail.neighbourBuildings
				for tile in rail.neighbourRails:
					ok = True
					for building in rail.neighbourBuildings:
						if building in tile.neighbourBuildings:
							ok = False
					if ok:
						okTiles.append(tile)
				#print rail.position, "ok", okTiles
			
				rail.tileValue = sum([2**i for i in range(6) if rail.neighbours[i] in okTiles])
			else:
	    		#print rail.position, "plain"
				rail.tileValue = sum([2**i for i in range(6) if rail.neighbours[i] in rail.neighbourRails or
									 							rail.neighbours[i].kind == "Station" or
									 							rail.neighbours[i] in rail.neighbourIndustries])
	    	
			self.g.set(rail.position, 64 + rail.tileValue)
		return rail.tileValue != oldVal





	def oldUpdateTileValue(self, rail, neighbours):
		### Store old value
		oldVal = rail.tileValue
		
		### Recalculate neighbouring rails
		rail.neighbours = neighbours
		rail.neighbourRails = [tile for tile in rail.neighbours if tile.kind == "Rail"]
		#print rail.position, "rail.neighbourRails", rail.neighbourRails
		
		### First check if we have no neighbours
		if rail.neighbourRails == []:
			rail.tileValue = 17
			#print rail.position, 1
			return rail.tileValue != oldVal
		
		### Then if there are any industries around
		#if [tile for tile in rail.neighbours if tile.kind == "Rail"]
		
		### Then if we have three or more (which means station tile)
		if len(rail.neighbourRails) > 2:
			rail.tileValue = 16
			#print rail.position, 2
			return rail.tileValue != oldVal
			
		### Over
		if rail.neighbours[TOP].kind == "Rail":
			### Lower left
			if rail.neighbours[BOTTOMLEFT].kind == "Rail":
				rail.tileValue = 22
			### Lower right
			elif rail.neighbours[BOTTOMRIGHT].kind == "Rail":
				rail.tileValue = 23
			else:
				rail.tileValue = 18
			return rail.tileValue != oldVal
			
		
		### Bottom
		if rail.neighbours[BOTTOM].kind == "Rail":
			### Top left
			if rail.neighbours[TOPLEFT].kind == "Rail":
				rail.tileValue = 27
			### Bottom
			elif rail.neighbours[TOPRIGHT].kind == "Rail":
				rail.tileValue = 26
			else:
				rail.tileValue = 18
			return rail.tileValue != oldVal


		
		### Top right
		if rail.neighbours[TOPRIGHT].kind == "Rail":
			### Top left
			if rail.neighbours[TOPLEFT].kind == "Rail":
				rail.tileValue = 25
			### Bottom
			elif rail.neighbours[BOTTOM].kind == "Rail":
				rail.tileValue = 26
			else:
				rail.tileValue = 19
			return rail.tileValue != oldVal
			
		### Bottom right
		if rail.neighbours[BOTTOMRIGHT].kind == "Rail":
			### Bottom left
			if rail.neighbours[BOTTOMLEFT].kind == "Rail":
				rail.tileValue = 24
			### Bottom
			elif rail.neighbours[TOP].kind == "Rail":
				rail.tileValue = 23
			else:
				rail.tileValue = 20
			return rail.tileValue != oldVal
			
		
		### Top left
		if rail.neighbours[TOPLEFT].kind == "Rail":
			### Top left
			if rail.neighbours[TOPRIGHT].kind == "Rail":
				rail.tileValue = 25
			### Bottom
			elif rail.neighbours[BOTTOM].kind == "Rail":
				rail.tileValue = 27
			else:
				rail.tileValue = 20
			return rail.tileValue != oldVal
			
		### Bottom left
		if rail.neighbours[BOTTOMLEFT].kind == "Rail":
			### Bottom left
			if rail.neighbours[TOP].kind == "Rail":
				rail.tileValue = 22
			### Bottom
			elif rail.neighbours[BOTTOMRIGHT].kind == "Rail":
				rail.tileValue = 24
			else:
				rail.tileValue = 19
			return rail.tileValue != oldVal
			
		return  rail.tileValue != oldVal


		 
		