import pygame
from pygame import locals
from dialogs import HighScoreDialog
from route import Route

### Scroll speed (w 31,h 32 is one tile)
SCROLL_SPEED_W = 31
SCROLL_SPEED_H = 32	
		
SCROLL_MARGIN = 50

from pgu import gui

class GUI:
	def __init__(self, windowSize, graphics, game):
		self.w, self.h = windowSize
		self.graphics = graphics
		self.game = game
		
		self.app = self.graphics.app
		
		self.mainContainer = gui.Container(width=200,height=400, align=-1, valign=-1)
		
		self.sidebar = gui.Container(width=200,height=400, align=-1, valign=-1)
		#self.requiredResourcesContainer = gui.Container(width=200,height=400, align=-1, valign=-1)
		self.requiredResourceLabels = [gui.Label("aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa"),
									   gui.Label("aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa"),
									   gui.Label("aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa"),
									   gui.Label("aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa"),
									   gui.Label("aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa")]
		for l in self.requiredResourceLabels[1:]:
			l.value = "* "
							   
		#self.buttonTable = gui.Table(width=200,height=300, align=-1, valign=-1)
		self.moneyLabel = gui.Label("Money = " + str(self.game.player.money))
		self.timeLabel = gui.Label(str(self.game.timeLeft))
		self.citySizeLabel = gui.Label("1000")
		
		#self.gameImageContainer  = gui.Container(width=-1,height=-1, align=-1, valign=-1)
		self.sourceIndustryImage = gui.Image("../media/sourceIndustry.png")
		self.targetIndustryImage = gui.Image("../media/targetIndustry.png")
		#self.sourceIndustryLabel = gui.Label("Resource(amount)")
		self.targetIndustryTypeLabel = gui.Label("Industry Type")
		self.targetIndustryTypeHelpLabel = gui.Label("You need to increase your city size before you can build a station here")
		self.targetIndustryLabel = gui.Label("self.style.width, self.style.height = self.font.size(self.value)")
		#self.needBiggerCity = gui.Label("You need to upgrade your city size first!")
		#self.app.init(self.gameImageContainer)
		
		self.stationIcons = []
		
		self.okImage = gui.Image("../media/ok.png")
		self.notOkImage = gui.Image("../media/notOk.png")
		
		self.setupMainGameGUI()
		
	def setupMainGameGUI(self):

		image = gui.Image("../media/sidebar03.png")

			
		self.sidebar.add(image, 0, 0)
		self.sidebar.add(self.moneyLabel, 23, 13)
		self.sidebar.add(self.timeLabel, 40, 87)
		self.sidebar.add(self.citySizeLabel, 40, 45)

		
		self.mainContainer.add(self.sidebar, 0, 0)
		self.app.init(self.mainContainer)
		#self.app.init(self.sidebar)


	def update(self):
		### Money Label
		self.moneyLabel.value = "$  " + str(self.game.player.money)
		self.timeLabel.value = str(self.game.timeLeft)
		### City size label
		cityTotal = str(sum([c.size for c in self.game.player.cities]))
		indTotal = str(len(self.game.player.industries))
		self.citySizeLabel.value = " " + indTotal + " / " + cityTotal
		
		
		### Images over selected industry and it's target
		if self.game.selectedIndustry is not None:
			x, y = self.graphics.g.tile_to_screen(self.game.selectedIndustry.position)
			if self.sourceIndustryImage not in self.mainContainer.widgets:
				self.mainContainer.add(self.sourceIndustryImage, x-5, y+15)
			if self.game.selectedIndustry.targetIndustry is not None:
				x, y = self.graphics.g.tile_to_screen(self.game.selectedIndustry.targetIndustry.position)	
				if self.targetIndustryImage not in self.mainContainer.widgets:
					self.mainContainer.add(self.targetIndustryImage, x-5, y+15)
					self.mainContainer.add(self.targetIndustryLabel, x-5, y-15)
					self.targetIndustryLabel.value = self.game.selectedIndustry.producedResource
		else:
			if self.sourceIndustryImage in self.mainContainer.widgets:
				self.mainContainer.remove(self.sourceIndustryImage)
			if self.targetIndustryImage in self.mainContainer.widgets:
				self.mainContainer.remove(self.targetIndustryImage)
				self.mainContainer.remove(self.targetIndustryLabel)

		self.app.init(self.mainContainer)
	
	def handleKeyScrolling(self, key):
		
		### Handle key scrolling
		if key == locals.K_LEFT:
			self.graphics.scroll(-SCROLL_SPEED_W, 0)		
		if key == locals.K_RIGHT:
			self.graphics.scroll(SCROLL_SPEED_W, 0)
		if key == locals.K_UP:
			self.graphics.scroll(0, -SCROLL_SPEED_H)
		if key == locals.K_DOWN:
			self.graphics.scroll(0, SCROLL_SPEED_H)

		self.updateIcons()
		
	def handleMousePointer(self):
		
		if pygame.mouse.get_focused():
			x,y = pygame.mouse.get_pos() 
		
			### Scrolling, this time with mouse
			if x < SCROLL_MARGIN:
				self.graphics.scroll(-SCROLL_SPEED_W, 0)
				
			if x > self.w - SCROLL_MARGIN:
				self.graphics.scroll(SCROLL_SPEED_W, 0)	
			if y < SCROLL_MARGIN:
				self.graphics.scroll(0, -SCROLL_SPEED_H)	
			if y > self.h - SCROLL_MARGIN:
				self.graphics.scroll(0, SCROLL_SPEED_H)		
			self.updateIcons()
		
			### Mouse hover effects
			tx,ty = self.graphics.g.screen_to_tile((x,y))
			try:
				tile = self.game.world.map[(tx,ty)]
			except:
				return
			
			if tile.isIndustry:
				
				index = 1
				for res in tile.requirements.keys():
					if self.requiredResourceLabels[index] in self.mainContainer.widgets:
						self.mainContainer.remove(self.requiredResourceLabels[index])
					self.mainContainer.add(self.requiredResourceLabels[index], x+20, y+10+20*index)
					self.requiredResourceLabels[index].value = "Requires: " + res
					index += 1

				if self.requiredResourceLabels[index] in self.mainContainer.widgets:
					self.mainContainer.remove(self.requiredResourceLabels[index])
				self.requiredResourceLabels[index].value = "Will buy resources for " + str(tile.purchaseValue)
				self.mainContainer.add(self.requiredResourceLabels[index], x+20, y+10+20*index)

				
				self.targetIndustryTypeLabel.value = tile.kind
				#self.targetIndustryTypeLabel.value = tile.kind
				if self.targetIndustryTypeLabel not in self.mainContainer.widgets:
					self.mainContainer.add(self.targetIndustryTypeLabel, x+20, y-10)
					self.mainContainer.add(self.targetIndustryTypeHelpLabel, x+20, y+10)
				else:
					self.mainContainer.remove(self.targetIndustryTypeLabel)
					self.mainContainer.remove(self.targetIndustryTypeHelpLabel)
					self.mainContainer.add(self.targetIndustryTypeLabel, x+20, y-10)
					self.mainContainer.add(self.targetIndustryTypeHelpLabel, x+20, y+10)
			
				if tile.owner is None:
					if len(self.game.player.industries) < sum([c.size for c in self.game.player.cities]):
						self.targetIndustryTypeHelpLabel.value = "(LMB) $" + str(tile.buyCost) + ": Build station"
					else:
						self.targetIndustryTypeHelpLabel.value = "City too small."
				elif tile.owner == self.game.player:
					if self.game.selectedIndustry is None:
						if tile.kind == "City":
							cityTotal = str(1000 * sum([c.size for c in self.game.player.cities])**2)
							self.targetIndustryTypeHelpLabel.value = "(RMB) $" + cityTotal +": Increase city size."
						else:
							self.targetIndustryTypeHelpLabel.value = "(LMB) Start transport route."
					else:
						self.targetIndustryTypeHelpLabel.value = "(LMB) End transport route."
					
			else:
				if self.targetIndustryTypeLabel in self.mainContainer.widgets:
					self.mainContainer.remove(self.targetIndustryTypeLabel)
					self.mainContainer.remove(self.targetIndustryTypeHelpLabel)	
				
				for label in self.requiredResourceLabels:
					if label in self.mainContainer.widgets:
						self.mainContainer.remove(label)

			
			
			if self.game.selectedIndustry is not None and \
					tile is not self.game.selectedIndustry and \
					tile.isIndustry and \
					tile.owner == self.game.player:
				if self.game.selectedIndustry.producedResource in tile.requirements.keys():
					if self.okImage not in self.mainContainer.widgets:
						self.mainContainer.add(self.okImage, x-20, y-20)
					else:
						self.mainContainer.remove(self.okImage)
						self.mainContainer.add(self.okImage, x-20, y-20)
				else:
					if self.notOkImage not in self.mainContainer.widgets:
						self.mainContainer.add(self.notOkImage, x-20, y-20)
					else:
						self.mainContainer.remove(self.notOkImage)
						self.mainContainer.add(self.notOkImage, x-20, y-20)
			else:
				if self.okImage in self.mainContainer.widgets:
					#print "hej"
					self.mainContainer.remove(self.okImage)
				if self.notOkImage in self.mainContainer.widgets:
					self.mainContainer.remove(self.notOkImage)
					
			self.app.init(self.mainContainer)
			#except:
			#	print "failed"
			
			
	def placeStationIcon(self, industry):		
		x, y = self.graphics.g.tile_to_screen(industry.position)
		stationImage = gui.Image("../media/greendot.png")
		self.stationIcons.append((industry, stationImage))
		print "######## ", x, y
		self.mainContainer.add(stationImage, x, y)
		
	def updateIcons(self):
		for industry, image in self.stationIcons:
			if image in self.mainContainer.widgets:
				self.mainContainer.remove(image)
			x, y = self.graphics.g.tile_to_screen(industry.position)
			self.mainContainer.add(image, x,y)
		
		
		