from player import Player
from graphics import Graphics
from graphics import TILEMAP_MAX
from graphics import TILEMAP_MIN
from game import Game
from gui import GUI
from dialogs import HighScoreDialog

import pygame
from pygame import locals

from pgu import gui
#from pgu import Image
import pgu.timer

X, Y = 0, 1

### Create main game modules
w, h = 640, 480
w, h = 800, 600
#w, h = 1024, 768


### Set the timer so that the game runs at a specific FPS
FPS = 30
timer = pgu.timer.Timer(FPS)
#timer = pgu.timer.Speedometer() # Uncomment to get unfixed fps



class MainMenu:
	def __init__(self):
		self.app = gui.Desktop(width=700,height=460)
		self.app.connect(gui.QUIT,self.app.quit,None)
		### Set title
		pygame.display.set_caption("Steam Empire")
	
		bg = gui.Image("../Media/menuBackground.png")
		
		self.res = gui.Select(value=(800,600))
		self.res.add("640x480",(640,480))
		self.res.add("800x600",(800,600))
		self.res.add("1024x768",(1024,768))
		
		
		e = gui.Button("New Game", width=150)
		e.connect(gui.CLICK,self.startGame,None)
		
		c = gui.Table(width=400,height=424)	
		c.tr()
		c.td(bg)
		c2 = gui.Table()
		c.td(c2)
		
		c2.td(gui.Label("Set Resolution"))
		c2.tr()
		c2.td(self.res,colspan=3)
		c2.tr()
		#c2.td(e)
		
		c2.td(e, align=-1)

		hsd = HighScoreDialog()
		e = gui.Button("View Highscores", width=150)
		e.connect(gui.CLICK, hsd.open, None)
		c2.tr()
		c2.td(e)

		e = gui.Button("Quit", width=150)
		e.connect(gui.CLICK, self.app.quit, None)
		c2.tr()
		c2.td(e, align=-1)
		self.app.add(c, 10,10)
		self.app.run(c)
		#self.screen = pygame.display.set_mode((640,480),locals.SWSURFACE)
		#self.loop()
		

	def loop(self):
		#app = gui.App()
		#gui.init(widget=widget)
		self.quit = False
		while not self.quit:
			for e in pygame.event.get():
				if e.type is locals.QUIT: self.quit = True
				if e.type is locals.KEYDOWN and e.key == locals.K_ESCAPE: self.quit = True
				self.app.event(e)
				
			self.app.paint(self.screen)
			pygame.display.flip()
					
	def startGame(self,e):
		w,h = self.res.value
		self.quit = True
		self.app.quit()
		main(w,h)

def main(w=640,h=480):
	
	player = Player(4000)
	graphics = Graphics(w, h, player) #ugly, the graphics shouldn't need to know about player
	game = Game(graphics, player)
	ggui = GUI((w,h), graphics, game)
	game.gui = ggui
	startTime = pygame.time.get_ticks()
	endTime = startTime + 10 * 1000 #(s * 1000 = ms)
	game.timeLeft = startTime - endTime
	### Loop until the application wants to quit or the user presses ESCAPE
	quit = False
	frame = 0
	while not quit:
		frame += 1
		for event in pygame.event.get():
			#print event
			
			### Handle Quitting
			if event.type == locals.QUIT:
				quit = True
			if event.type == locals.KEYDOWN and event.key == locals.K_ESCAPE: 
				quit = True
			
						
			### Handle mouse input
			SIDEBAR_WIDTH, SIDEBAR_HEIGHT = 150, 400
			if event.type == locals.MOUSEBUTTONDOWN:
				if event.button == 1:
					if event.pos[X] > SIDEBAR_WIDTH or event.pos[Y] > SIDEBAR_HEIGHT:
						(x,y) = graphics.getTileFromScreenCoords(event.pos)
						if x >= TILEMAP_MIN and x <= TILEMAP_MAX-1 and y >= TILEMAP_MIN and y <= TILEMAP_MAX-1:
							print "oooo", x, y
							game.handleMouseClick((x,y))
				if event.button == 3:
					if event.pos[X] > SIDEBAR_WIDTH or event.pos[Y] > SIDEBAR_HEIGHT:
						(x,y) = graphics.getTileFromScreenCoords(event.pos)
						if x >= TILEMAP_MIN and x <= TILEMAP_MAX-1 and y >= TILEMAP_MIN and y <= TILEMAP_MAX-1:
							game.handleRightMouseClick((x,y))

			### Keyboard scrolling
			if event.type == locals.KEYDOWN:
				ggui.handleKeyScrolling(event.key)
			
			### Let the graphics engine handle any events
			graphics.app.event(event)

		#gui.handleMouseHover()
		if not frame % 5: 
			game.update()
		ggui.update()
		ggui.handleMousePointer()
		graphics.render()
		
		"""			
		### Handle mouse input
		if event.type == locals.MOUSEBUTTONDOWN:
			if event.button == 1:
				try:
					(x,y) = graphics.getTileFromScreenCoords(event.pos)
					if x > 0 and y > 0:
						game.handleMouseClick((x,y))
				except Exception, e: 
					print e
					pass
		"""		
				

		### Let the graphics engine handle any events
		#graphics.app.event(event)

		### Keep the game at a steady FPS
		timer.tick()
		#print "FPS:", timer.fps # for use when fps is unfixed

		### Handle time
		game.timeLeft = (endTime - pygame.time.get_ticks()) / 1000
		if endTime - pygame.time.get_ticks() < 0:
			quit = True
			print "No time left"
	
	
	### Back to the main menu
	pygame.display.quit()
	print "Game ended successfully!"
	menu = MainMenu()

DO_PROFILING = 0

if DO_PROFILING:
	import profile
	profile.run('main()')
else:
	menu = MainMenu()
